package pl.net.labkeeper.ejb.securities.fixedincome;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.*;

import javax.annotation.Resource;
import javax.ejb.*;
import javax.validation.*;
import javax.validation.constraints.*;

import pl.net.labkeeeper.ejb.securities.fixedincome.*;

@Stateful(name = "AccruedInterestBean", mappedName = "ejb/securities/fixedincome/AccruedInterest")
@LocalBean
public class AccruedInterest implements AccruedInterestRemote,
    AccruedInterestLocal, Serializable {

  public AccruedInterest() {

  }

  @NotNull
  @DecimalMin("0.01")
  @Digits(fraction = 2, integer = 10)
  private BigDecimal nominalValue;

  public BigDecimal getCouponRate() {
    return couponRate;
  }

  public void setCouponRate(BigDecimal couponRate) {
    this.couponRate = couponRate;
  }

  public BigDecimal getIndexRate() {
    return indexRate;
  }

  public void setIndexRate(BigDecimal indexRate) {
    this.indexRate = indexRate;
  }

  public Date getCalculationDate() {
    return calculationDate;
  }

  public void setCalculationDate(Date calculationDate) {
    this.calculationDate = calculationDate;
  }

  public List<Date> getCouponPeriod() {
    return couponPeriod;
  }

  public void setCouponPeriod(List<Date> couponPeriod) {
    this.couponPeriod = couponPeriod;
  }

  public DayCountConvention getDayCountConvention() {
    return dayCountConvention;
  }

  public void setDayCountConvention(DayCountConvention dayCountConvention) {
    this.dayCountConvention = dayCountConvention;
  }

  public Validator getValidator() {
    return validator;
  }

  public void setValidator(Validator validator) {
    this.validator = validator;
  }

  @NotNull
  @DecimalMin("0.00")
  @Digits(fraction = 8, integer = 10)
  private BigDecimal couponRate;

  @NotNull
  @DecimalMin("0.00000001")
  @Digits(fraction = 8, integer = 10)
  private BigDecimal indexRate;

  @NotNull
  private Date calculationDate;

  @NotNull
  @Size(min = 3, max = 3)
  private List<Date> couponPeriod;

  @NotNull
  private DayCountConvention dayCountConvention;

  @Resource
  private transient Validator validator;

  @Override
  public BigDecimal calculate() throws ConstraintViolationException {

    Set<ConstraintViolation<AccruedInterest>> violations = validator
        .validate(this);
    if (violations.size() > 0) {
      Set<ConstraintViolation<?>> toThrow = new HashSet<ConstraintViolation<?>>(
          violations);
      throw new ConstraintViolationException(toThrow);
    }

    return null;
  }

  public BigDecimal getNominalValue() {
    return nominalValue;
  }

  public void setNominalValue(BigDecimal nominalValue) {
    this.nominalValue = nominalValue;
  }

}
